package cn.fw.hermes.controller.api.service;

import static cn.fw.common.web.util.ExceptionHandler.handleException;
import static cn.fw.common.web.util.ResultBuilder.failureWithMessage;
import static cn.fw.common.web.util.ResultBuilder.success;
import static cn.fw.hermes.common.constant.MessageStr.QUERY_FAILURE;
import static cn.fw.hermes.common.constant.MessageStr.SAVE_FAILURE;

import cn.fw.data.base.domain.common.Message;
import cn.fw.hermes.sdk.api.para.AccountCondition;
import cn.fw.hermes.sdk.api.ImAccountService;
import cn.fw.hermes.sdk.api.result.AccountResult;
import cn.fw.hermes.service.biz.AccountBizService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * @Author: Chenery
 * @Date: 2021/2/2 15:56
 */
@RestController
@Slf4j
@RequiredArgsConstructor
@Validated
public class ImAccountApiServiceImpl implements ImAccountService {

    private final AccountBizService accountBizService;


    @Override
    @PostMapping("/api/account/register")
    public Message<AccountResult> userRegister(@RequestBody AccountCondition accountCondition) {
        final String msg = "用户注册[/api/account/register]";
        try {
            log.info("{}: param[{}]", msg, accountCondition);
            AccountResult accountResult = accountBizService.userRegister(accountCondition);
            return success(accountResult,
                    data -> log.info("{} 成功: result:[{}]", msg, accountResult));
        } catch (Exception ex) {
            handleException(ex, e -> log.error("{} 失败: result:[{}]", msg, e.getMessage()));
            return failureWithMessage(SAVE_FAILURE);
        }
    }


    @Override
    @GetMapping("/api/im/register")
    public Message<AccountResult> register(@RequestParam("userId") Long userId,
            @RequestParam("userName") String userName,
            @RequestParam(value = "faceUrl", required = false) String faceUrl,
            @RequestParam("isStaff") boolean isStaff) {
        final String msg = "用户注册[/api/im/register]";
        try {
            AccountCondition condition = new AccountCondition();
            condition.setStaff(isStaff);
            condition.setUserId(userId);
            condition.setUserName(userName);
            condition.setFaceUrl(faceUrl);
            log.info("{}: param[{}]", msg, condition);
            AccountResult accountResult = accountBizService.userRegister(condition);
            return success(accountResult,
                    data -> log.info("{} 成功: result:[{}]", msg, accountResult));
        } catch (Exception ex) {
            handleException(ex, e -> log.error("{} 失败: result:[{}]", msg, e.getMessage()));
            return failureWithMessage(SAVE_FAILURE);
        }

    }

    @Override
    @GetMapping("/api/im/getSig")
    public Message<AccountResult> getSig(@RequestParam("userId") Long userId,
            @RequestParam(value = "expire", required = false) Long expire,
            @RequestParam("isStaff") boolean isStaff) {
        final String msg = "获取用户签名[/api/im/getSig]";
        try {
            log.info("{} params[{},{},{}]", msg, userId, expire, isStaff);
            String sig = accountBizService.getSig(userId, isStaff);
            AccountResult result = new AccountResult();
            result.setUserSig(sig);
            return success(result, data -> log.info("{} 成功 ：result[{}]", msg, result));
        } catch (Exception ex) {
            handleException(ex, e -> log.error("{} 失败 : result[{}]", msg, e));
            return failureWithMessage(SAVE_FAILURE);
        }

    }

    @Override
    @GetMapping("/api/im/kick")
    public Message<Boolean> kick(@RequestParam("userId") Long userId) {
        final String msg = "踢用户下线[/api/im/kick]";
        try {
            log.info("{} params[{}]", msg, userId);
            Boolean result = accountBizService.kick(userId);
            return success(result, data -> log.info("{}成功 ：result[{}]", msg, result));
        } catch (Exception ex) {
            handleException(ex, e -> log.error("{} 失败 ：result[{}]", msg, e));
            return failureWithMessage(SAVE_FAILURE);
        }

    }

    @Override
    @GetMapping("/api/im/getIdentifier")
    public Message<String> getIdentifier(@RequestParam("userId") Long userId,
            @RequestParam("isStaff") boolean isStaff) {
        final String msg = "获取IM用户识别码[/api/im/getIdentifier]";
        try {
            log.info("{} params[{},{}]", msg, userId, isStaff);
            String identifierCode = accountBizService.queryIdentifierCode(userId, isStaff);
            log.info("{} 成功 ：result[{}]", msg, identifierCode);
            return success(identifierCode);
        } catch (Exception e) {
            handleException(e, ex -> log.error("{} 失败 ：result[{}]", msg, e));
            return failureWithMessage(QUERY_FAILURE);
        }

    }


}
